@@ -14,7 +14,7 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
  * 
- * $Id: config.h,v 1.4 2005/05/04 16:34:20 astra Exp $
+ * $Id: config.h,v 1.1.1.1 2005/09/21 14:22:14 exodusd Exp $
  */
 
 #ifndef _CONFIG_H
@@ -33,6 +33,10 @@
 #define SOCKET_READ_SIZE 65536
 #define HTTP_MAX_BODY  20000000
 
+#define WEBDAV_CLASS2
+
+#define READ_ONCE 80
+
 #define OBJ_COAL   "/kernel/securesocket.pike"
 #define OBJ_SCOAL  "/kernel/securesocket.pike"
 #define OBJ_NNTP   "/net/nntp.pike"
@@ -46,7 +50,7 @@
 #define OBJ_TELNET "/net/telnet.pike"
 #define OBJ_XMLRPC "/net/xmlrpc.pike"
 
-#define STEAM_VERSION "1.6.3"
+#define STEAM_VERSION "2.0.0"
 
 #define CLASS_PATH "classes/"
 
