@@ -16,7 +16,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  *
- * $Id: xml_sax.c,v 1.1.1.1 2005/02/23 14:47:22 cvs Exp $
+ * $Id: xml_sax.c,v 1.2 2005/10/06 09:23:51 astra Exp $
  */
 
 /*
@@ -64,7 +64,7 @@
 //#define SAX_DEBUG 1
 
 #include "global.h"
-RCSID("$Id: xml_sax.c,v 1.1.1.1 2005/02/23 14:47:22 cvs Exp $");
+RCSID("$Id: xml_sax.c,v 1.2 2005/10/06 09:23:51 astra Exp $");
 
 #ifdef fp
 #undef fp
@@ -1083,18 +1083,32 @@ static void f_parse_xml(INT32 args)
 static void f_parse_html(INT32 args)
 {
   xmlDocPtr   doc = NULL;
+  char * encoding = "utf-8";
+  struct pike_string *encode_data = NULL;
   
+  if ( args == 1 ) {
+      if ( ARG(1).type != T_STRING ) 
+	  Pike_error("Incorrect type for argument 0: expected string (encoding)\n");
+      encode_data = ARG(1).u.string;
+      encoding = encode_data->str;
+  }
+  // do nothing
+  if ( THIS->input_data->len == 0 )
+    push_int(0);
+
   switch (THIS->parsing_method) {
       case PARSE_PUSH_PARSER:
         Pike_error("Push parser not implemented yet. Please bug grendel@caudium.net to implement it.");
         
       case PARSE_MEMORY_PARSER:
-        doc = htmlSAXParseDoc(THIS->input_data->str, NULL, THIS->sax, NULL);
+        htmlHandleOmittedElem(1);
+	doc=htmlSAXParseDoc(THIS->input_data->str, encoding, THIS->sax, NULL);
         break;
 
       case PARSE_FILE_PARSER:
-        doc = htmlSAXParseFile(THIS->input_data->str, NULL, THIS->sax, NULL);
-        break;
+        htmlHandleOmittedElem(1);
+	doc=htmlSAXParseFile(THIS->input_data->str, "utf-8", THIS->sax, NULL);
+	break;
   }
   if ( doc != NULL )
     xmlFreeDoc(doc);
@@ -1164,7 +1178,7 @@ int _init_xml_sax(void)
 
   ADD_FUNCTION("create", f_create,
                tFunc(tOr(tString, tObj) tObj tOr(tMapping, tVoid) tOr(tMixed, tVoid) tOr(tInt, tVoid), tVoid), 0);
-  ADD_FUNCTION("parse", f_parse_html, tFunc(tVoid, tInt), 0);
+  ADD_FUNCTION("parse", f_parse_html, tFunc(tOr(tString,tVoid), tInt), 0);
   ADD_FUNCTION("getLineNumber", f_getLineNumber, tFunc(tVoid, tInt), 0);
   ADD_FUNCTION("getColumnNumber", f_getColumnNumber, tFunc(tVoid, tInt), 0);
   
