@@ -14,7 +14,7 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
  * 
- * $Id: xslt.c,v 1.3 2005/04/08 16:23:38 alain Exp $
+ * $Id: xslt.c,v 1.3 2005/10/08 14:36:47 astra Exp $
  */
 
 #include <string.h>
@@ -46,6 +46,7 @@
 #include <fcntl.h>
 
 #include "xslt.h"
+#include "xml_dom.h"
 
 //#define XSLT_DEBUG 1
 
@@ -122,15 +123,41 @@ void f_create(INT32 args)
 //!
 void f_set_xml_data(INT32 args)
 {
+    struct pike_string* xml_data;
+
     if ( args != 1 )
 	Pike_error("XSLT.Parser()->set_xml_data: Expected one argument.\n");
-    if ( Pike_sp[-args].type != T_STRING )
-	Pike_error("XSLT.Parser()->set_xml_data: Expected string.\n");
+
     if( THIS->xml != NULL ) 
 	free_string(THIS->xml);
+    if ( THIS->err_str != NULL ) {
+	free_string(THIS->err_str);
+	THIS->err_str = NULL;
+    }
 
-    THIS->xml = Pike_sp[-args].u.string;
-    add_ref(THIS->xml);
+    switch ( ARG(1).type ) {
+    case T_STRING:
+	xmlSetGenericErrorFunc(THIS, xml_error);
+	xml_data = Pike_sp[-args].u.string;
+	//add_ref(THIS->xml);
+	THIS->doc = xmlParseMemory(xml_data->str, xml_data->len);
+	if ( THIS->err_str != NULL ) {
+	    // only free doc at this point
+	    xmlFreeDoc(THIS->doc);
+	    Pike_error(THIS->err_str->str);
+	}
+	break;
+    case T_OBJECT:
+	CHECK_DOM(Pike_sp[0-1].u.object);
+	add_ref(Pike_sp[0-1].u.object);
+	THIS->doc = OBJ2_DOM(Pike_sp[0-1].u.object)->domDoc;
+	break;
+    default:
+	Pike_error("XSLT.Parser()->set_xml_data: Expected string|object.\n");
+    }
+
+
+    
     pop_n_elems(args);    
 }
 
@@ -256,7 +283,7 @@ static void f_run( INT32 args )
     if ( args != 1 || Pike_sp[-args].type != T_OBJECT )
 	Pike_error("XSLT.Parser->run(): requires XSL Stylesheet parameter.\n");
 
-    if ( THIS->xml == NULL ) {
+    if ( THIS->doc == NULL ) {
 	Pike_error("XML input not set correctly.\n");
     }
     if ( THIS->err_str != NULL ) {
@@ -268,20 +295,15 @@ static void f_run( INT32 args )
     
     DEBUG_XSLT("Running XSL transformation\n");
 
+#if 0
     xmlstr = THIS->xml->str;
-
-    xmlSetGenericErrorFunc(THIS, xml_error);
     if ( THIS->xml->len == 0 ) {
 	Pike_error("No XML code given - cannot transform.\n");
 	return;
     }
-    doc = xmlParseMemory(THIS->xml->str, THIS->xml->len);
+#endif
+    doc = THIS->doc;
 
-    if ( THIS->err_str != NULL ) {
-	// only free doc at this point
-	xmlFreeDoc(doc);
-	Pike_error(THIS->err_str->str);
-    }
     if ( doc == NULL ) {
 	Pike_error("Unable to parse xml source.\n");
     }
@@ -334,10 +356,13 @@ static void f_run( INT32 args )
 	    
 	    str = (const xmlChar*) sind.u.string->str;
 	    // namespaces are bad
-	    if ( xmlStrchr(str, ':') ) 
-		vars[tmpint++] = (const char*) "supressed";
-	    else
+	    if ( xmlStrchr(str, ':') ) {
+		Pike_error("Invalid Parameter %s in namespace format !",
+			   str);
+	    }
+	    else {
 		vars[tmpint++] = sind.u.string->str;
+	    }
 	    vars[tmpint++] = value;
 	    params[varcount++] = value;
 	    if ( varcount > MAX_PARAMS )
@@ -748,28 +773,25 @@ void pike_module_init( void )
     DEBUG_XSLT("RegisterInputCallbacks()");
     // old libxml (pre 2.6)
 
-    fprintf(stderr, "========================\nXSLT Init: xml=%s, xslt=%s\n",LIBXML_DOTTED_VERSION, LIBXSLT_DOTTED_VERSION);
-
 #if LIBXML_VERSION < 20600
     if ( xmlRegisterInputCallbacks(_include_match, _include_open, _include_read, _include_close) == -1 )
       fprintf(stderr, "Error registering input callbacks !");
 #else
-    // new one!
-    fprintf(stderr, "XSLT Init: version > 2.06 external entity loading\n");
     defaultLoader = xmlGetExternalEntityLoader();
     xmlSetExternalEntityLoader(steamExternalEntityLoader);
     if ( xmlGetExternalEntityLoader() != steamExternalEntityLoader )
       fprintf(stderr, "XSLT: Failed to set external entity loader !\n");
 #endif
 
+    _init_xml_dom();
+
     start_new_program();
     ADD_STORAGE(xslt_storage);
     set_init_callback(init_xslt_storage);
     set_exit_callback(free_xslt_storage);
     ADD_FUNCTION2("create", f_create, tFunc(tNone,tVoid), 0,
 		 OPT_EXTERNAL_DEPEND|OPT_SIDE_EFFECT);
-    ADD_FUNCTION2("set_xml_data", f_set_xml_data, tFunc(tStr,tVoid),0,
-		  OPT_EXTERNAL_DEPEND|OPT_SIDE_EFFECT);
+    ADD_FUNCTION2("set_xml_data", f_set_xml_data, tFunc(tOr(tStr, tObj), tVoid),0, OPT_EXTERNAL_DEPEND|OPT_SIDE_EFFECT);
     ADD_FUNCTION2("set_variables", f_set_variables, 
 		 tFunc(tMapping,tVoid), 0,
 		 OPT_EXTERNAL_DEPEND|OPT_SIDE_EFFECT);
