@@ -16,9 +16,9 @@
 #include "stringutils.h"
 
 
-typedef enum { blank, para, list, pre, done } status_t;
 typedef enum { start, end, next } list_t;
 
+static lexer_t Lexer;
 
 char *disambiguate(char *link_text);
 void prepare_status(status_t new);
@@ -29,13 +29,6 @@ void list_tag(char c, list_t type);
 /* A signal sent to make_list() */
 #define CLOSE_TAGS "close tags"
 
-/* Define a lexer object, available or functions in this file */
-typedef struct {
-    status_t status;
-} lexer_t;
-static lexer_t Lexer;
-
-
 
 
 /*
@@ -132,8 +125,6 @@ char *disambiguate(char *link_text)
     return link_text;
 }
 
-
-
 /*
  * heading()
  *
@@ -184,12 +175,15 @@ void heading(int new_level, boolean start)
  */
 void paragraph(char *stuff)
 {
-    prepare_status(para);
-    
     if (Lexer.status == para) {
         plaintext(stuff);
     }
+    else if ( Lexer.status == list ) {
+        output("<br/>");
+        plaintext(stuff);
+    }
     else {
+	prepare_status(para);
         output( "<p>");
         Lexer.status = para;
         plaintext(stuff);
@@ -204,16 +198,11 @@ void make_def(char* stuff)
   output("<dd>");
 }
 
-void tag(char* stuff)
+void force_br(void) 
 {
-  output("&lt;");
-  stuff = &stuff[1];
-  stuff[strlen(stuff)-1] = '&';
-  output(stuff);
-  output("gt;");
+  output("<br />");
 }
 
-
 /*
  * preformat()
  *
@@ -317,6 +306,7 @@ void __image(char *link)
 void hr (void)
 {
     prepare_status(blank);
+    Lexer.status = blank;
     output( "\n<hr>\n");
 }
 
@@ -381,8 +371,8 @@ void prepare_status(status_t new)
  */
 void blank_line(void)
 {
-    prepare_status(blank);
-    Lexer.status = blank;
+  prepare_status(blank);
+  Lexer.status = blank;
 }
 
 
@@ -417,9 +407,6 @@ void make_list(char *new)
         Lexer.status = blank;
         return;
     }
-    else {
-        Lexer.status = list;
-    }
 
     new_list = duplicate_cstring(new);
     new_len = strlen(new_list);
@@ -511,8 +498,44 @@ void list_tag(char c, list_t type)
     }
 }
 
+void make_table(char *table)
+{
+    char* tdef;
+    int   slen;
+    char*  ptr;
 
+    if ( strings_equal(table, "{|") ) {
+	output("<table><tr>\n");
+	return;
+    }
+    tdef = &table[2];
+    output("<table %s><tr>\n", tdef); 
+}
 
+void make_tr(char *table)
+{
+    char*    tdef;
+    int   i, slen;
+    char*     ptr;
+    
+    if ( strings_equal(table, "|-") ) {
+	output("</tr><tr>\n");
+	return;
+    }
+    tdef = strstr(table, "=");
+    if ( tdef == NULL ) {
+	output("</tr><tr>\n");
+	return;
+    }
+    slen = strlen(table);
+    i = 1;
+    while ( i < slen && table[i] == '-' )
+	i++;
+    
+    if ( i >= slen - 1 )
+	i = 0;
+    output("</tr><tr %s>\n", &table[i]); 
+}
 
 /*
  * close_tags()
@@ -530,6 +553,10 @@ void close_tags(char *tags)
     }
 }
 
+void eof()
+{
+  prepare_status(blank);
+}
 
 
 
@@ -542,3 +569,10 @@ void __barelink(char *link)
 {
     output("<a class=\"external\" href=\"%s\">%s</a>", link, link);
 }
+
+void table_cells(char* td)
+{
+    output("<td>");
+    output(td);
+    output("</td>");
+}
