@@ -24,8 +24,19 @@ void paragraph(char *text);
 void plaintext(char *text);
 void preformat(void);
 void wikilink(char *link);
+void tag(char *tagstr);
 
 void linkInternal(char *link);
 void annotationInternal(char *ann);
 
+typedef enum { blank, para, list, pre, done } status_t;
+
+
+/* Define a lexer object, available or functions in this file */
+typedef struct {
+    status_t status;
+} lexer_t;
+
+void prepare_status(status_t new);
+
 #endif
