@@ -1,4 +1,4 @@
-/* Copyright (C) 2000-2004  Thomas Bopp, Thorsten Hampel, Ludger Merkens
+/* Copyright (C) 2000-2005  Thomas Bopp, Thorsten Hampel, Ludger Merkens
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -24,6 +24,8 @@
 #include "yyexternals.h"
 
 static struct program  *wiki_program;
+extern lexer_t Lexer;
+
 
 #define safe_push_text(_txt_) if ((_txt_)) push_text((_txt_)); else push_int(0)
 #define THIS ((wiki_store*)Pike_fp->current_storage)
@@ -45,7 +47,7 @@ char *yyinbuffer, *yyoutbuffer;
 static char* callback_api[] =
 {
   "annotationWiki", "linkInternalWiki", "imageWiki", "hyperlinkWiki",
-  "barelinkWiki", "embedWiki", "pikeWiki",
+  "barelinkWiki", "embedWiki", "pikeWiki", "tagWiki",
 };
 
 void bold(void)
@@ -60,8 +62,6 @@ void bold(void)
   }
 }
 
-
-
 /*
  * italic()
  *
@@ -69,7 +69,6 @@ void bold(void)
  */
 void italic(void)
 {
-
     if ( THIS->italic ) { 
        output( "</em>");
        THIS->italic = FALSE;
@@ -80,6 +79,20 @@ void italic(void)
     }
 }
 
+void tag(char* stuff)
+{
+  add_ref(THIS->obj);
+  push_object(THIS->obj);
+  add_ref(THIS->fp);
+  push_object(THIS->fp);
+
+  push_text(stuff);
+  CB_CALL(cb_tag, 3);
+  CB_OUT_RESULT();
+}
+
+
+
 
 void annotationInternal(char* ann_text)
 {
@@ -258,6 +271,7 @@ void f_wiki2html(INT32 args)
   init_lexer();
   yyin = NULL;
   yylex();
+  prepare_status(blank);
   pop_n_elems(args);
 
   html = get_output(THIS->outStart);
